package main

import (
	"context"
	"log"
	"os"
	"os/signal"
	"syscall"

	"tildegit.org/tjp/gus/contrib/cgi"
	guslog "tildegit.org/tjp/gus/contrib/log"
	"tildegit.org/tjp/gus/gemini"
)

func main() {
	// GET TLS files from the environment
	certfile, keyfile := envConfig()

	// build a TLS configuration suitable for gemini
	tlsconf, err := gemini.FileTLS(certfile, keyfile)
	if err != nil {
		log.Fatal(err)
	}

	// make use of a CGI request handler
	cgiHandler := cgi.CGIDirectory("/cgi-bin", "./cgi-bin")

	// add stdout logging to the request handler
	handler := guslog.Requests(os.Stdout, nil)(cgiHandler)

	// set up signals to trigger graceful shutdown
	ctx, stop := signal.NotifyContext(context.Background(), syscall.SIGINT, syscall.SIGHUP)
	defer stop()

	// run the server
	server, err := gemini.NewServer(ctx, tlsconf, "tcp4", ":1965", handler)
	if err != nil {
		log.Fatal(err)
	}
	defer server.Close()

	if err := server.Serve(); err != nil {
		log.Fatal(err)
	}
}

func envConfig() (string, string) {
	certfile, ok := os.LookupEnv("SERVER_CERTIFICATE")
	if !ok {
		log.Fatal("missing SERVER_CERTIFICATE environment variable")
	}

	keyfile, ok := os.LookupEnv("SERVER_PRIVATEKEY")
	if !ok {
		log.Fatal("missing SERVER_PRIVATEKEY environment variable")
	}

	return certfile, keyfile
}
